#libraries used
library(Seurat)
library(scCustomize)
library(ggplot2)
library(scater)
library(scds)
library(fgsea)
library(future)
library(dplyr)
library(patchwork)
library(viridis)
plan("multisession", workers = 10)

#Create a color palette, used in a few figures
cols <- c("#5050FF", "#CE3D32", "#749B58", "#F0E685", "#466983", "#BA6338", "#5DB1DD", "#802268", "#6BD76B", "#D595A7", "#924822", "#837B8D", "#C75127", "#D58F5C", "#7A65A5", "#3B1B53", "#CDDEB7", "#AE1F63", "#99CC00", "#A9A9A9", "#CC9900", "#00CC33", "#0099CC", "#0A47FF", "#FFD147", "#991A00", "#996600", "#809900", "#008099", "#003399", "#1A0099", "#990080", "#FF1463", "#E377C2", "#FF0000", "#FF00CC", "#FFFF00", "#CCFF00", "#00FFFF", "#9900CC", "#AE1FCC")

#Single cell Colorectal cancer dataset loading and processing, GSE178341, download data from https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE178341
sce.data <- Read10X_h5("GSE178341_crc10x_full_c295v4_submit.h5", use.names = TRUE, unique.features = TRUE)
sce.meta <- read.csv("GSE178341_crc10x_full_c295v4_submit_metatables.csv", h=T, row.names=1, sep=",")
sce <- CreateSeuratObject(counts = sce.data, project = "scColon_GSE178341", meta.data = sce.meta)
sce.meta <- read.csv("crc10x_full_c295v4_submit_cluster.csv", h=T, row.names=1, sep=",")
sce <- AddMetaData(sce, sce.meta)
sce <- SCTransform(sce, verbose = FALSE)
sce <- RunPCA(sce, features = VariableFeatures(object = sce))
sce <- FindNeighbors(sce, dims = 1:10)
sce <- FindClusters(sce)
sce <- RunUMAP(sce, dims = 1:10)

#Figure S2A
DimPlot_scCustom(sce, pt.size = 2, label = T, label.size = 8, raster = F, group.by = "SCT_snn_res.0.8")

#Figure S2B
Idents(sce) <- "clTopLevel"
#Highlight all seven lineages - left panel
Cluster_Highlight_Plot(sce, cluster_name = c("Strom", "Plasma", "Mast", "B", "TNKILC", "Myeloid", "Epi"), highlight_color = c("#F6222E", "#FEAF16", "#3283FE", "#16FF32", "#C55A11", "#FE00FA", "#5A5156"), background_color = "lightgray", raster = F, label = T, pt.size = 2)
#Highlight each of the seven lineages - right panels
Cluster_Highlight_Plot(sce, cluster_name = c("Strom"), highlight_color = c("#F6222E"), background_color = "lightgray", raster = F, label = T, pt.size = 2)
Cluster_Highlight_Plot(sce, cluster_name = c("Plasma"), highlight_color = c("#FEAF16"), background_color = "lightgray", raster = F, label = T, pt.size = 2)
Cluster_Highlight_Plot(sce, cluster_name = c("Mast"), highlight_color = c("#3283FE"), background_color = "lightgray", raster = F, label = T, pt.size = 2)
Cluster_Highlight_Plot(sce, cluster_name = c("B"), highlight_color = c("#16FF32"), background_color = "lightgray", raster = F, label = T, pt.size = 2)
Cluster_Highlight_Plot(sce, cluster_name = c("TNKILC"), highlight_color = c("#C55A11"), background_color = "lightgray", raster = F, label = T, pt.size = 2)
Cluster_Highlight_Plot(sce, cluster_name = c("Myeloid"), highlight_color = c("#FE00FA"), background_color = "lightgray", raster = F, label = T, pt.size = 2)
Cluster_Highlight_Plot(sce, cluster_name = c("Epi"), highlight_color = c("#5A5156"), background_color = "lightgray", raster = F, label = T, pt.size = 2)

#Assigning hybrid cell identity to cells that express two markers: EPCAM and CD163
Idents(sce, WhichCells(object=sce, expression=EPCAM > 0 & CD163 > 0, slot='data')) <-"EPCAM+ CD163+"
Idents(sce, WhichCells(object=sce, expression=EPCAM > 0 & CD163 > 0, slot='data', invert = T)) <-"Other"

#Add the identity based on 2 markers to metadata with a column name 'hybridcells2'
sce$hybridcells2 <- Idents(sce)

#Count number of hybrid cells based on two markers
table(sce@meta.data$hybridcells2)

#Figure 1A, dimplot highlighting cells that express two markers
Cluster_Highlight_Plot(sce, cluster_name = c("EPCAM+ CD163+"), highlight_color = c("#00B0F0"), background_color = "lightgray", raster = F, label = F, pt.size = 2) + NoLegend()

#Assigning hybrid cell identity based on stringent criteria, cells that express four marker: EPCAM, KRT8, CD163, and CD14
Idents(sce, WhichCells(object=sce, expression=EPCAM > 0 & CD163 > 0 & CD14 > 0 & KRT8 > 0, slot='data')) <-"EPCAM+ KRT8+ CD163+ CD14+"
Idents(sce, WhichCells(object=sce, expression=EPCAM > 0 & CD163 > 0 & CD14 > 0 & KRT8 > 0, slot='data', invert = T)) <-"Other"

#Add the identity based on 4 markers to metadata with a column name 'hybridcells4'
sce$hybridcells4 <- Idents(sce)
#Count number of hybrid cells based on four markers
table(sce@meta.data$hybridcells4)

#Figure 1B, dimplot highlighting cells that express four markers
Cluster_Highlight_Plot(sce, cluster_name = c("EPCAM+ KRT8+ CD163+ CD14+"), highlight_color = c("#7030A0"), background_color = "lightgray", raster = F, label = F, pt.size = 2) + NoLegend()

#Doublet scoring, convert to single cell experiment
sc <- as.SingleCellExperiment(sce)

#Doublet scoring calculations using three different approaches
sc = cxds(sc)
sc = bcds(sc)
sc = cxds_bcds_hybrid(sc)

#Add doublet scores to the Seurat object
sce$hybridscore <- sc@colData@listData[["hybrid_score"]]
sce$cxdsscore <- sc@colData@listData[["cxds_score"]]
sce$bcdsscore <- sc@colData@listData[["bcds_score"]]

#Figure 1C (Top), violin plot of hybrid scores before filtering for doublet cells. 
Idents(sce) <- "clTopLevel"
VlnPlot_scCustom(sce, features = c("hybridscore"), plot_boxplot = T)

#Create a new Seurat object by keeping cells with hybrid score of less than 0.5
sce_nd <- subset(sce, hybridscore < 0.5)

#Count number of hybrid cells based on four markers after removing doublets
table(sce_nd@meta.data$hybridcells4)

#Figure 1C (Bottom), violin plot of hybrid scores after filtering for doublet cells. 
VlnPlot_scCustom(sce_nd, features = c("hybridscore"), plot_boxplot = T)

#Figure 1D, feature plot showing expression of each gene
FeaturePlot(sce_nd, features = C("EPCAM", "KRT8", "KRT18", "KRT19", "CD163", "CD14", "CD68", "PTPRC"), raster = F, pt.size = 2, ncol = 2)

#Figure S2C, dimplot of 19 cell types
DimPlot_scCustom(sce, pt.size = 2, label = T, raster = F, group.by = "clMidwayPr")

#Figure S2D, dimplot of all 87 cell types
DimPlot_scCustom(sce, pt.size = 2, label = T, raster = F, group.by = "cl295v11SubFull")

#Figure 1E
Idents(sce_nd) <- "clMidwayPr"
Idents(sce_nd, WhichCells(object=sce_nd, expression=EPCAM > 0 & CD163 > 0 & CD14 > 0 & KRT8 > 0, slot='data')) <-"Hybrid cells"
Clustered_DotPlot(sce_nd, features = c("CD163", "CD68", "CD14", "PTPRC", "KRT8", "EPCAM", "KRT19", "KRT18"), flip = T)

#Create a new Seurat object by keeping myeloid cells, subset of sce_nd
sce_nd_m <- subset(sce_nd, clTopLevel=="Myeloid")

#Figure S2E, left showing all 10 cell types
Idents(sce_nd_m) <- "cl295v11SubFull"
Cluster_Highlight_Plot(sce_nd_m, cluster_name = c("cM01 (Monocyte)", "cM02 (Macrophage-like)", "cM03 (DC1)", "cM04 (DC2)", "cM05 (DC2 C1Q+)", "cM06 (DC IL22RA2)", "cM07 (pDC)", "cM08 (AS-DC)", "cM09 (mregDC)", "cM10 (Granulocyte)"), highlight_color = c("#F6222E", "#5A5156", "#B00068", "#FE00FA", "#16FF32", "#FEAF16", "#1CFFCE", "#90AD1C", "#D86ECC", "#3283FE"), background_color = "lightgray", raster = F) + NoLegend()

#Figure S2E, right 10 panels showing each cell types
Cluster_Highlight_Plot(sce_nd_m, cluster_name = "cM01 (Monocyte)", highlight_color = "#F6222E", background_color = "lightgray", raster = F)
Cluster_Highlight_Plot(sce_nd_m, cluster_name = "cM02 (Macrophage-like)", highlight_color = "#5A5156", background_color = "lightgray", raster = F) + NoLegend()
Cluster_Highlight_Plot(sce_nd_m, cluster_name = "cM03 (DC1)", highlight_color = "#B00068", background_color = "lightgray", raster = F) + NoLegend()
Cluster_Highlight_Plot(sce_nd_m, cluster_name = "cM04 (DC2)", highlight_color = "#FE00FA", background_color = "lightgray", raster = F) + NoLegend()
Cluster_Highlight_Plot(sce_nd_m, cluster_name = "cM05 (DC2 C1Q+)", highlight_color = "#16FF32", background_color = "lightgray", raster = F) + NoLegend()
Cluster_Highlight_Plot(sce_nd_m, cluster_name = "cM06 (DC IL22RA2)", highlight_color = "#FEAF16", background_color = "lightgray", raster = F) + NoLegend()
Cluster_Highlight_Plot(sce_nd_m, cluster_name = "cM07 (pDC)", highlight_color = "#1CFFCE", background_color = "lightgray", raster = F) + NoLegend()
Cluster_Highlight_Plot(sce_nd_m, cluster_name = "cM08 (AS-DC)", highlight_color = "#90AD1C", background_color = "lightgray", raster = F) + NoLegend()
Cluster_Highlight_Plot(sce_nd_m, cluster_name = "cM09 (mregDC)", highlight_color = "#D86ECC", background_color = "lightgray", raster = F) + NoLegend()
Cluster_Highlight_Plot(sce_nd_m, cluster_name = "cM10 (Granulocyte)", highlight_color = "#3283FE", background_color = "lightgray", raster = F) + NoLegend()

#Figure 1F (right)
Idents(sce_nd_m, WhichCells(object=sce_nd_m, expression=EPCAM > 0 & CD163 > 0 & CD14 > 0 & KRT8 > 0, slot='data')) <-"Hybrid cells"
Cluster_Highlight_Plot(sce_nd_m, cluster_name = c("Hybrid cells"), highlight_color = c("#7030A0"), background_color = "lightgray", raster = F, label = F, pt.size = 2) + NoLegend()

#Figure 1G
DotPlot_scCustom(sce_nd_m, features = c("CD163", "CD14", "EPCAM", "KRT8"), scale = F, flip_axes = F)

#Figure 1H & 1I
#Create a new Seurat object by keeping cells derived from tumor fraction, subset of sce_nd
sce_nd_t <- subset(sce_nd, SPECIMEN_TYPE=="T")

#Add the identity to cells that express all four markers (THC), or EPCAM and KRT8 (EPCAM+ KRT8+), or CD14 and CD163 (CD14+ CD163+), abd rest of the cells (Other) as follows
Idents(sce_nd_t) <- "SCT_snn_res.0.8"
Idents(sce_nd_t, WhichCells(object=sce_nd_t, expression=EPCAM > 0 & CD163 > 0 & CD14 > 0 & KRT8 > 0, slot='data')) <-"THC"
Idents(sce_nd_t, WhichCells(object=sce_nd_t, expression=EPCAM > 0 & CD163 > 0 & CD14 > 0 & KRT8 > 0, slot='data',  invert = T)) <-"Other"
Idents(sce_nd_t, WhichCells(object=sce_nd_t, expression=EPCAM > 0 & KRT8 > 0, slot='data')) <-"EPCAM+ KRT8+"
Idents(sce_nd_t, WhichCells(object=sce_nd_t, expression=CD14 > 0 & CD163 > 0, slot='data')) <-"CD14+ CD163+"
Idents(sce_nd_t, WhichCells(object=sce_nd_t, expression=EPCAM > 0 & CD163 > 0 & CD14 > 0 & KRT8 > 0, slot='data')) <-"THC"

#Add the above identities to metadata with a column name 'class4'
sce_nd_t$class4 <- sce_nd_t@active.ident

#Perform a differential gene expression analysis between THC and EPCAM+ KRT8+ cells and save the results as Table S1
Epi_vs_THC <- FindMarkers(sce_nd_t, ident.1='THC', ident.2='EPCAM+ KRT8+')
write.table(Epi_vs_THC, file = "Table S1.txt", sep = "\t")

#Perform a differential gene expression analysis between THC and EPCAM+ KRT8+ cells and save the results as Table S2
Macro_vs_THC <- FindMarkers(sce_nd_t, ident.1='THC', ident.2='CD14+ CD163+')
write.table(Macro_vs_THC, file = "Table S2.txt", sep = "\t")

#Open the files written above in Excel, calculate -log10(pvalue) in one column and upload the file to https://goedhart.shinyapps.io/VolcaNoseR/

#Settings used in VolcaNoseR.
#https://goedhart.shinyapps.io/VolcaNoseR/?data=3;;avg_log2FC;minus_log10_pvalue;Gene&vis=8;0.8;-0,0;5.8;significant;manh&can=33;;;&layout=;;TRUE;-10,10;;X;2000;1600&color=5;Grey,%20Blue,%20Red,%20Black&label=;;;;;TRUE;24;90;60;16;&

#Figure 1J, pick top 5 genes from each of Epi_vs_THC and Macro_vs_THC and create a dotplot
genes <- c("CD163", "CD14", "TYROBP", "FCER1G", "AIF1", "EPCAM", "KRT8", "LGALS4", "KRT18", "PHGR1")
DotPlot_scCustom(sce_nd_t, features = genes, scale = F, flip_axes = F)

#Figure 2A
Idents(sce_nd) <- "hybridcells4"
Cluster_Highlight_Plot(sce_nd, cluster_name = c("EPCAM+ KRT8+ CD163+ CD14+"), highlight_color = c("#7030A0"), background_color = "lightgray", raster = F, label = F, pt.size = 2, split.by = "SPECIMEN_TYPE", split_seurat = T) + NoLegend()

#Data for 2B, the figure is made in Excel
table(sce_nd@meta.data$SPECIMEN_TYPE, sce_nd@meta.data$clTopLevel)

#Figure 2C
Idents(sce_nd) <- "clTopLevel"
Cluster_Highlight_Plot(sce_nd, cluster_name = "Myeloid", highlight_color = "#FE00FA", background_color = "lightgray", pt.size = 2, raster = F, split.by = "SPECIMEN_TYPE", split_seurat = T)

#Data for Figure 2D
table(sce_nd@meta.data$SPECIMEN_TYPE)

#Figure S3A
Idents(sce_nd) <- "hybridcells4"
Cluster_Highlight_Plot(sce_nd, cluster_name = "EPCAM+ KRT8+ CD163+ CD14+", highlight_color = "#7030A0", background_color = "lightgray", raster = F, split.by = "PatientTypeID", split_seurat = T, num_columns = 10)

#Data for Figure S3B and Figure 2E
table(sce_nd@meta.data$PatientTypeID, sce_nd@meta.data$hybridcells4)

#Create a new Seurat object by keeping hybrid cells, subset of sce_nd
sce_nd_h <- subset(sce_nd, hybridcells4=="EPCAM+ KRT8+ CD163+ CD14+")

#Perform a differential gene expression analysis between THC and NHC and save the results as Table S3
Idents(sce_nd_h) <- "SPECIMEN_TYPE"
hybrid_NvsT <- FindMarkers(sce_nd_h, ident.1 = "T", ident.2 = "N")
write.table(hybrid_TvsN, file = "Table S3.txt", sep = "\t")

#Open the file written above in Excel, calculate -log10(pvalue) in one column and upload the file to https://goedhart.shinyapps.io/VolcaNoseR/

#Settings used in VolcaNoseR.
#https://goedhart.shinyapps.io/VolcaNoseR/?data=3;;avg_log2FC;minus_log10_pvalue;Gene&vis=8;0.8;-0,0;5.8;significant;manh&can=33;;;&layout=;;TRUE;-10,10;;X;2000;1600&color=5;Grey,%20Blue,%20Red,%20Black&label=;;;;;TRUE;24;90;60;16;&

#Figure 2G, split in two figures
DotPlot_scCustom(sce_nd_h, features = c("SPP1", "CXCL8", "FCGR3A", "PLAUR", "CTSL", "HIF1A", "C15orf48", "SOD2", "CCL3", "CD44", "LDHA"), scale = F, flip_axes = T, x_lab_rotate = T)
DotPlot_scCustom(sce_nd_h, features = c("ZG16", "F13A1", "DNASE1L3", "FOLR2", "CST3", "CD36", "CFD", "IGHA1", "SLC40A1", "SEPP1", "JCHAIN", "PDK4", "CLEC10A", "SERPINF1", "LYVE1", "LTC4S", "MVB12B", "VCAM1", "GUCA2A", "BCAM", "GUCA2B", "OTOP2"), scale = F, flip_axes = T, x_lab_rotate = T)

#Figure 2H
DotPlot_scCustom(sce_nd_h, features = c("CD163", "CD14", "EPCAM", "KRT8", "KRT18"), scale = F, scale.max = 100, scale.min = 0, flip_axes = T)

#Figure S4A
FeaturePlot_scCustom(sce_nd_h, features = "CD44", raster = F, pt.size = 2, split.by = "SPECIMEN_TYPE")

#Figure S4B
FeaturePlot_scCustom(sce_nd_h, features = "SPP1", raster = F, pt.size = 2, split.by = "SPECIMEN_TYPE")

#Perform functional gene set enrichment analysis
#Download gmt file 'h.all.v2023.2.Hs.symbols.gmt' from MSigDB at https://www.gsea-msigdb.org/gsea/msigdb 
hallmark <- gmtPathways("h.all.v2023.2.Hs.symbols.gmt")
hybrid_TvsN <- hybrid_TvsN %>% arrange(desc(avg_log2FC))
fold_changes <- hybrid_TvsN$avg_log2FC
names(fold_changes) <- rownames(hybrid_TvsN)
gsea_hyb <- fgsea(pathways = hallmark, stats = fold_changes, eps = 0.0, minSize = 15, maxSize = 500)

#Figure 2I
topPathwaysUp <- gsea_hyb[ES > 0][head(order(pval), n=10), pathway]
plotGseaTable(hallmark[topPathwaysUp], fold_changes, gsea_hyb, gseaParam=0.5)

#Figure 2J
plotEnrichment(hallmark[["HALLMARK_EPITHELIAL_MESENCHYMAL_TRANSITION"]], fold_changes) + labs(title="EMT; pval = 3.00e-08, padj = 2.40e-07")

#Figure S5A
fgseaResTidy <- gsea_hyb %>%
  as_tibble() %>%
  arrange(desc(NES))

# Make table:
fgseaResTidy %>% 
  dplyr::select(-leadingEdge, -ES, -nMoreExtreme) %>% 
  arrange(padj) %>% 
  DT::datatable()

ggplot(fgseaResTidy, aes(reorder(pathway, NES), NES)) +
  geom_col(aes(fill=padj<0.05)) +
  coord_flip() +
  labs(x="Pathway", y="Normalized Enrichment Score",
       title="Hallmark pathways NES from GSEA") + 
  theme_minimal()

#Figure S5B
plotEnrichment(hallmark[["HALLMARK_TNFA_SIGNALING_VIA_NFKB"]], fold_changes) + labs(title="TNFA SIGNALING VIA NFKB; pval = 2.49E-24, padj = 1.19E-22")
plotEnrichment(hallmark[["HALLMARK_INFLAMMATORY_RESPONSE"]], fold_changes) + labs(title="INFLAMMATORY RESPONSE; pval = 3.49E-19, padj = 8.37E-18")
plotEnrichment(hallmark[["HALLMARK_KRAS_SIGNALING_UP"]], fold_changes) + labs(title="KRAS SIGNALING UP; pval = 0.00000000152, padj = 0.0000000243")
plotEnrichment(hallmark[["HALLMARK_ALLOGRAFT_REJECTION"]], fold_changes) + labs(title="ALLOGRAFT REJECTION; pval = 0.0000000132, padj = 0.00000015")
plotEnrichment(hallmark[["HALLMARK_HYPOXIA"]], fold_changes) + labs(title="HYPOXIA; pval = 0.0000000156, padj = 0.00000015")
plotEnrichment(hallmark[["HALLMARK_EPITHELIAL_MESENCHYMAL_TRANSITION"]], fold_changes) + labs(title="EPITHELIAL MESENCHYMAL TRANSITION; pval = 0.0000000301, padj = 0.000000241")
plotEnrichment(hallmark[["HALLMARK_IL6_JAK_STAT3_SIGNALING"]], fold_changes) + labs(title="IL6 JAK STAT3 SIGNALING; pval = 0.0000000975, padj = 0.000000668")
plotEnrichment(hallmark[["HALLMARK_MTORC1_SIGNALING"]], fold_changes) + labs(title="MTORC1 SIGNALING; pval = 0.000000179, padj = 0.00000107")
plotEnrichment(hallmark[["HALLMARK_INTERFERON_GAMMA_RESPONSE"]], fold_changes) + labs(title="INTERFERON GAMMA RESPONSE; pval = 0.000000645, padj = 0.00000344")
plotEnrichment(hallmark[["HALLMARK_IL2_STAT5_SIGNALING"]], fold_changes) + labs(title="IL2 STAT5 SIGNALING; pval = 0.00000156, padj = 0.0000075")
plotEnrichment(hallmark[["HALLMARK_COMPLEMENT"]], fold_changes) + labs(title="COMPLEMENT; pval = 0.00000579, padj = 0.0000253")
plotEnrichment(hallmark[["HALLMARK_APOPTOSIS"]], fold_changes) + labs(title="APOPTOSIS; pval = 0.0000809, padj = 0.000324")
plotEnrichment(hallmark[["HALLMARK_COAGULATION"]], fold_changes) + labs(title="COAGULATION; pval = 0.000592, padj = 0.00218")
plotEnrichment(hallmark[["HALLMARK_ANGIOGENESIS"]], fold_changes) + labs(title="ANGIOGENESIS; pval = 0.00104, padj = 0.00355")
plotEnrichment(hallmark[["HALLMARK_GLYCOLYSIS"]], fold_changes) + labs(title="GLYCOLYSIS; pval = 0.00189, padj = 0.00604")
plotEnrichment(hallmark[["HALLMARK_CHOLESTEROL_HOMEOSTASIS"]], fold_changes) + labs(title="CHOLESTEROL HOMEOSTASIS; pval = 0.00837, padj = 0.0251")
plotEnrichment(hallmark[["HALLMARK_TGF_BETA_SIGNALING"]], fold_changes) + labs(title="TGF BETA SIGNALING; pval = 0.00905, padj = 0.0256")
plotEnrichment(hallmark[["HALLMARK_P53_PATHWAY"]], fold_changes) + labs(title="P53 PATHWAY; pval = 0.0155, padj = 0.0413")
plotEnrichment(hallmark[["HALLMARK_UNFOLDED_PROTEIN_RESPONSE"]], fold_changes) + labs(title="UNFOLDED PROTEIN RESPONSE; pval = 0.019, padj = 0.0479")

#Figure S6, all data for A to E is provided in table S4. The analysis was done in GraphPad Prism.

#Create a new Seurat object by keeping tumor cells, subset of sce_nd
sce_nd_t <- subset(sce_nd, SPECIMEN_TYPE=="T")

#Figure 2K
Cluster_Highlight_Plot(sce_nd_t, cluster_name = "EPCAM+ KRT8+ CD163+ CD14+", highlight_color = "#7030A0", background_color = "lightgray", raster = F, pt.size = 2, split.by = "TumorStage", split_seurat = T)

#Create a new Seurat object by keeping myeloid cells, subset of sce_nd_t
sce_nd_tm <- subset(sce_nd_t, clTopLevel=="Myeloid")

#Data for Figure 2L, figure made in Excel
table(sce_nd_tm@meta.data$TumorStage, sce_nd_tm@meta.data$hybridcells4)

#Spatial Colon cancer and normal datasets
#Download the data from https://www.10xgenomics.com/datasets/human-colon-preview-data-xenium-human-colon-gene-expression-panel-1-standard
path <- "Path/to/Xenium_V1_hColon_Non_diseased_Add_on_FFPE_outs"
spx_n <- LoadXenium(path, fov = "fov")
spx_n <- subset(spx_n, subset = nCount_Xenium > 0)
spx_n <- SCTransform(spx_n, assay = "Xenium")
spx_n <- RunPCA(spx_n, npcs = 30, features = rownames(spx_n))
spx_n <- FindNeighbors(spx_n, reduction = "pca", dims = 1:30)
spx_n <- FindClusters(spx_n, resolution = 0.8)
spx_n <- RunUMAP(spx_n, dims = 1:30)
DefaultBoundary(spx_n[["fov"]]) <- "segmentation"

#Figure S7A (left)
ImageDimPlot(spx_n, cols = cols, size = 0.75, axes = T)

#Figure S7A (right)
DimPlot_scCustom(spx_n, reduction = "umap", colors_use = cols, raster = F, group.by = "SCT_snn_res.0.8")


#Label hybrid cells and other cells
Idents(spx_n, WhichCells(object=spx_n, expression=EPCAM > 0 & CD163 > 0 & CD14 > 0 & KRT8 > 0, slot='data')) <-"Hybrid Cells"
Idents(spx_n, WhichCells(object=spx_n, expression=EPCAM > 0 & CD163 > 0 & CD14 > 0 & KRT8 > 0, slot='data', invert = T)) <-"Other"
spx_n$hybridcells <- Idents(spx_n)

#Label hybrid cells, epithelial, macrophge/monocyte, and other cells
Idents(spx_n) <- "SCT_snn_res.0.8"
Idents(spx_n, WhichCells(object=spx_n, expression=EPCAM > 0 & CD163 > 0 & CD14 > 0 & KRT8 > 0, slot='data')) <-"THC"
Idents(spx_n, WhichCells(object=spx_n, expression=EPCAM > 0 & CD163 > 0 & CD14 > 0 & KRT8 > 0, slot='data',  invert = T)) <-"Other"
Idents(spx_n, WhichCells(object=spx_n, expression=EPCAM > 0 & KRT8 > 0, slot='data')) <-"EPCAM+ KRT8+"
Idents(spx_n, WhichCells(object=spx_n, expression=CD14 > 0 & CD163 > 0, slot='data')) <-"CD14+ CD163+"
Idents(spx_n, WhichCells(object=spx_n, expression=EPCAM > 0 & CD163 > 0 & CD14 > 0 & KRT8 > 0, slot='data')) <-"THC"
spx_n$class4 <- spx_n@active.ident

#Export and write the file to disc
exp_class <- FetchData(spx_n, vars = c("EPCAM", "KRT8", "CD14", "CD163", "hybrid4", "class4"))
write.table(exp_class, file = "spx_n_expression and class.txt", sep = "\t")

#The four classes were relabeled into 16 classes in Excel based on the expression from file "spx_n_expression and class.txt". EPCAM only, CD163 only, CD14 only, CD163 only, EPCAM & KRT8 only, CD14 & CD163 only, EPACM & CD14 only, EPCAM & CD163 only, KRT8 & CD14 only KRT8 & CD163 only, EPCAM KRT8 & CD14 only EPCAM KRT8 & CD163 only, CD14 CD163 & EPCAM only, CD14 CD163 & KRT8 only, Other, THC
#Save the file with two columns, 1=cell names and 2=class16 columns only, as csv file, "spx_n_class16.meta.csv", and add as meta data.
#Save another file with two columns, 1=cell_id (cell names), 2=group (class16), file name=spx_n_class16.group.csv for import in Xenium explorer
sce.meta <- read.csv("spx_n_class16.meta.csv", h=T, row.names=1, sep=",")
spx_n <- AddMetaData(spx_n, sce.meta)

#Figure 3A
Idents(spx_n) <- "hybridcells"
Cluster_Highlight_Plot(spx_n, cluster_name = c("Hybrid Cells"), highlight_color = c("#7030A0"), background_color = "lightgray", raster = F, label = F, pt.size = 2) + NoLegend()

#Figure 3B
#Import "spx_n_class16.group.csv" in Xenum explorer where the same image and data is loaded. Select cell clusters: "THC", "EPCAM & KRT8 only", and "CD16 & CD163 only"
# Screenshot using Windows 'Snipping Tool', a full image (1X), Manually zoom to an area with all three types of cells and snip again at 10x, and again manually Zoom to 100x and snip. 

#Download the data from https://www.10xgenomics.com/datasets/human-colon-preview-data-xenium-human-colon-gene-expression-panel-1-standard
path <- "Path/to/Xenium_V1_hColon_Cancer_Add_on_FFPE_outs"
spx_c <- LoadXenium(path, fov = "fov")
spx_c <- subset(spx_c, subset = nCount_Xenium > 0)
spx_c <- SCTransform(spx_c, assay = "Xenium")
spx_c <- RunPCA(spx_c, npcs = 30, features = rownames(spx_c))
spx_c <- FindNeighbors(spx_c, reduction = "pca", dims = 1:30)
spx_c <- FindClusters(spx_c, resolution = 0.8)
spx_c <- RunUMAP(spx_c, dims = 1:30)
DefaultBoundary(spx_c[["fov"]]) <- "segmentation"

#Figure S7B (Left)
ImageDimPlot(spx_c, cols = cols, size = 0.75, axes = T)

#Figure S7B (Right)
DimPlot_scCustom(spx_c, reduction = "umap", colors_use = cols, raster = F, group.by = "SCT_snn_res.0.8")

#Label hybrid cells and other cells
Idents(spx_c, WhichCells(object=spx_c, expression=EPCAM > 0 & CD163 > 0 & CD14 > 0 & KRT8 > 0, slot='data')) <-"Hybrid Cells"
Idents(spx_c, WhichCells(object=spx_c, expression=EPCAM > 0 & CD163 > 0 & CD14 > 0 & KRT8 > 0, slot='data', invert = T)) <-"Other"
spx_c$hybridcells <- Idents(spx_c)

#Label hybrid cells, epithelial, macrophge/monocyte, and other cells
Idents(spx_c) <- "SCT_snn_res.0.8"
Idents(spx_c, WhichCells(object=spx_c, expression=EPCAM > 0 & CD163 > 0 & CD14 > 0 & KRT8 > 0, slot='data')) <-"THC"
Idents(spx_c, WhichCells(object=spx_c, expression=EPCAM > 0 & CD163 > 0 & CD14 > 0 & KRT8 > 0, slot='data',  invert = T)) <-"Other"
Idents(spx_c, WhichCells(object=spx_c, expression=EPCAM > 0 & KRT8 > 0, slot='data')) <-"EPCAM+ KRT8+"
Idents(spx_c, WhichCells(object=spx_c, expression=CD14 > 0 & CD163 > 0, slot='data')) <-"CD14+ CD163+"
Idents(spx_c, WhichCells(object=spx_c, expression=EPCAM > 0 & CD163 > 0 & CD14 > 0 & KRT8 > 0, slot='data')) <-"THC"
spx_c$class4 <- spx_c@active.ident

#Export and write the file to disc
exp_class <- FetchData(spx_c, vars = c("EPCAM", "KRT8", "CD14", "CD163", "hybrid4", "class4"))
write.table(exp_class, file = "spx_c_expression and class.txt", sep = "\t")

#The four classes were relabeled into 16 classes in Excel based on the expression from file "spx_c_expression and class.txt". EPCAM only, CD163 only, CD14 only, CD163 only, EPCAM & KRT8 only, CD14 & CD163 only, EPACM & CD14 only, EPCAM & CD163 only, KRT8 & CD14 only KRT8 & CD163 only, EPCAM KRT8 & CD14 only EPCAM KRT8 & CD163 only, CD14 CD163 & EPCAM only, CD14 CD163 & KRT8 only, Other, THC
#Save the file with two columns, 1=cell names and 2=class16 columns only, as csv file, "spx_c_class16.meta.csv", and add as meta data.
#Save another file with two columns, 1=cell_id (cell names), 2=group (class16), file name=spx_c_class16.group.csv for import in Xenium explorer
sce.meta <- read.csv("spx_c_class16.meta.csv", h=T, row.names=1, sep=",")
spx_c <- AddMetaData(spx_c, sce.meta)

#Figure 3C
Idents(spx_c) <- "hybridcells"
Cluster_Highlight_Plot(spx_c, cluster_name = c("Hybrid Cells"), highlight_color = c("#7030A0"), background_color = "lightgray", raster = F, label = F, pt.size = 2) + NoLegend()

#Figure 3D
#Import "spx_c_class16.group.csv" in Xenum explorer where the same image and data is loaded. Select cell clusters: "THC", "EPCAM & KRT8 only", and "CD16 & CD163 only"
# Screenshot using Windows 'Snipping Tool', a full image (1X), Manually zoom to an area with all three types of cells and snip again at 10x, and again manually Zoom to 100x and snip. 

#For differential gene expression analysis between spatial normal colon and cancer colon tissues, create and merge a Seurat object
counts_n <- spx_n@assays$Xenium$counts
counts_c <- spx_c@assays$Xenium$counts

sce_n <- CreateSeuratObject(counts = counts_n, project = "Normal")
sce_c <- CreateSeuratObject(counts = counts_c, project = "Cancer")

sce_cn <- merge(sce_n, y=sce_c)

options(future.globals.maxSize = 8000 * 1024^2)
plan("multisession", workers = 10)

sce_cn <- SCTransform(sce_cn, verbose = FALSE)
sce_cn <- RunPCA(sce_cn, features = VariableFeatures(object = sce_cn))
sce_cn <- FindNeighbors(sce_cn, dims = 1:10)
sce_cn <- FindClusters(sce_cn)
sce_cn <- RunUMAP(sce_cn, dims = 1:10)

Idents(sce_cn, WhichCells(object=sce_cn, expression=EPCAM > 0 & CD163 > 0 & CD14 > 0 & KRT8 > 0, slot='data')) <-"Hybrid Cells"
Idents(sce_cn, WhichCells(object=sce_cn, expression=EPCAM > 0 & CD163 > 0 & CD14 > 0 & KRT8 > 0, slot='data', invert = T)) <-"Other"
sce_cn$hybridcells <- Idents(sce_cn)

#Figure 3E
sce_cn_h <- subset(sce_cn, hybridcells=="Hybrid Cells")
sce_cn_h <- PrepSCTFindMarkers(sce_cn_h)
Tumor_vs_Normal <- FindMarkers(sce_cn_h, ident.1='Cancer', ident.2='Normal')
write.table(Tumor_vs_Normal, file = "Table S5.txt", sep = "\t")

#Open the files written above in Excel, calculate -log10(pvalue) in one column and upload the file to https://goedhart.shinyapps.io/VolcaNoseR/

#Settings
#https://goedhart.shinyapps.io/VolcaNoseR/?data=3;;avg_log2FC;minus_log10_pvalue;Gene&vis=8;0.8;-0,0;5.8;significant;manh&can=33;;;&layout=;;TRUE;-10,10;;X;2000;1600&color=5;Grey,%20Blue,%20Red,%20Black&label=;;;;;TRUE;24;90;60;16;&

#Figure 3F
DotPlot_scCustom(sce_cn_h, features = c("CD163", "CD14", "EPCAM", "KRT8"), scale = F, flip_axes = F)

#Figure 3G
ImageFeaturePlot(spx_c, features = c("CD14"))
ImageFeaturePlot(spx_c, features = c("KRT8"))
ImageFeaturePlot(spx_c, features = c("CD163"))
ImageFeaturePlot(spx_c, features = c("EPCAM"))

#Download the breast cancer single cell data and meta data from https://singlecell.broadinstitute.org/single_cell/study/SCP1039/a-single-cell-and-spatially-resolved-atlas-of-human-breast-cancers#study-download
counts <- Read10X('path/to/gse176078', gene.column = 2, cell.column = 1, unique.features = TRUE, strip.suffix = FALSE)
sce.meta <- read.csv("path/to/gse176078/Whole_miniatlas_meta.csv", h=T, row.names=1, sep=",")
sce_bc <- CreateSeuratObject(counts = counts, project = "scBreast")
sce_bc <- AddMetaData(sce_bc, sce.meta)
sce_bc <- SCTransform(sce_bc)
sce_bc <- RunPCA(sce_bc, features = VariableFeatures(object = sce_bc))
sce_bc <- FindNeighbors(sce_bc, dims = 1:10)
sce_bc <- FindClusters(sce_bc, resolution = 0.8)
sce_bc <- RunUMAP(sce_bc, dims = 1:10)

Idents(sce_bc, WhichCells(object=sce_bc, expression=EPCAM > 0 & CD163 > 0 & CD14 > 0 & KRT8 > 0, slot='data')) <-"Hybrid Cells"
Idents(sce_bc, WhichCells(object=sce_bc, expression=EPCAM > 0 & CD163 > 0 & CD14 > 0 & KRT8 > 0, slot='data', invert = T)) <-"Other"
sce_bc$hybridcells <- Idents(sce_bc)

#Doublet scoring, convert to single cell experiment
sc <- as.SingleCellExperiment(sce_bc)

#Doublet scoring calculations using three different approaches
sc = cxds(sc)
sc = bcds(sc)
sc = cxds_bcds_hybrid(sc)

#Add doublet scores to the Seurat object
sce_bc$hybridscore <- sc@colData@listData[["hybrid_score"]]
sce_bc$cxdsscore <- sc@colData@listData[["cxds_score"]]
sce_bc$bcdsscore <- sc@colData@listData[["bcds_score"]]

#Keep cells with hybrid score of less than 0.5
sce_bc_nd <- subset(sce_bc, hybridscore < 0.5)

#Figure 4A
DimPlot_scCustom(sce_bc_nd, colors_use = cols, pt.size = 2, group.by = "celltype_minor")

#Figure 4B
Idents(sce_bc_nd) <- "SCT_snn_res.0.8"
Idents(sce_bc_nd, WhichCells(object=sce_bc_nd, expression=EPCAM > 0 & CD163 > 0, slot='data')) <-"Hybrid Cells"
Cluster_Highlight_Plot(sce_bc_nd, cluster_name = c("Hybrid Cells"), highlight_color = c("#7030A0"), background_color = "lightgray", raster = F, label = F, pt.size = 2) + NoLegend()
Cluster_Highlight_Plot(sce_bc_nd, cluster_name = c("Monocyte"), highlight_color = c("#CE3D32"), background_color = "lightgray", raster = F, label = F, pt.size = 2) + NoLegend()
Cluster_Highlight_Plot(sce_bc_nd, cluster_name = c("Macrophage"), highlight_color = c("#5050FF"), background_color = "lightgray", raster = F, label = F, pt.size = 2) + NoLegend()

#Figure 4C
Idents(sce_bc_nd) <- "celltype_minor"
Idents(sce_bc_nd, WhichCells(object=sce_bc_nd, expression=EPCAM > 0 & CD163 > 0, slot='data')) <-"THC"
Clustered_DotPlot(sce_bc_nd, features = c("CD163", "CD68", "CD14", "PTPRC", "KRT8", "EPCAM", "KRT19", "KRT18"), flip = F)

#Figure S9
DimPlot_scCustom(sce_bc_nd, colors_use = cols, pt.size = 2, group.by = "celltype_major")

#Spatial breast cancer data, download from https://www.10xgenomics.com/datasets/xenium-ffpe-human-breast-with-custom-add-on-panel-1-standard
path <- "path/to/Xenium_V1_FFPE_Human_Breast_IDC_With_Addon_outs"
spx_bc <- LoadXenium(path, fov = "fov")
spx_bc <- subset(spx_bc, subset = nCount_Xenium > 0)
spx_bc <- SCTransform(spx_bc, assay = "Xenium")
spx_bc <- RunPCA(spx_bc, npcs = 30, features = rownames(spx_bc))
spx_bc <- FindNeighbors(spx_bc, reduction = "pca", dims = 1:30)
spx_bc <- FindClusters(spx_bc, resolution = 0.8)
spx_bc <- RunUMAP(spx_bc, dims = 1:30)
DefaultBoundary(spx_bc[["fov"]]) <- "segmentation"

#Figure S7C (Left)
ImageDimPlot(spx_bc, cols = cols, size = 0.75, axes = T)

#Figure S7C (Right)
DimPlot_scCustom(spx_bc, reduction = "umap", colors_use = cols, raster = F, group.by = "SCT_snn_res.0.8")

#Label hybrid cells and other cells
Idents(spx_bc, WhichCells(object=spx_bc, expression=EPCAM > 0 & CD163 > 0 & CD14 > 0 & KRT8 > 0, slot='data')) <-"Hybrid Cells"
Idents(spx_bc, WhichCells(object=spx_bc, expression=EPCAM > 0 & CD163 > 0 & CD14 > 0 & KRT8 > 0, slot='data', invert = T)) <-"Other"
spx_bc$hybridcells <- Idents(spx_bc)

#Label hybrid cells, epithelial, macrophge/monocyte, and other cells
Idents(spx_bc) <- "SCT_snn_res.0.8"
Idents(spx_bc, WhichCells(object=spx_bc, expression=EPCAM > 0 & CD163 > 0 & CD14 > 0 & KRT8 > 0, slot='data')) <-"THC"
Idents(spx_bc, WhichCells(object=spx_bc, expression=EPCAM > 0 & CD163 > 0 & CD14 > 0 & KRT8 > 0, slot='data',  invert = T)) <-"Other"
Idents(spx_bc, WhichCells(object=spx_bc, expression=EPCAM > 0 & KRT8 > 0, slot='data')) <-"EPCAM+ KRT8+"
Idents(spx_bc, WhichCells(object=spx_bc, expression=CD14 > 0 & CD163 > 0, slot='data')) <-"CD14+ CD163+"
Idents(spx_bc, WhichCells(object=spx_bc, expression=EPCAM > 0 & CD163 > 0 & CD14 > 0 & KRT8 > 0, slot='data')) <-"THC"
spx_bc$class4 <- spx_bc@active.ident

#Export and write the file to disc
exp_class <- FetchData(spx_bc, vars = c("EPCAM", "KRT8", "CD14", "CD163", "hybrid4", "class4"))
write.table(exp_class, file = "spx_bc_expression and class.txt", sep = "\t")

#The four classes were relabeled into 16 classes in Excel based on the expression from file "spx_bc_expression and class.txt". EPCAM only, CD163 only, CD14 only, CD163 only, EPCAM & KRT8 only, CD14 & CD163 only, EPACM & CD14 only, EPCAM & CD163 only, KRT8 & CD14 only KRT8 & CD163 only, EPCAM KRT8 & CD14 only EPCAM KRT8 & CD163 only, CD14 CD163 & EPCAM only, CD14 CD163 & KRT8 only, Other, THC
#Save the file with two columns, 1=cell names and 2=class16 columns only, as csv file, "spx_bc_class16.meta.csv", and add as meta data.
#Save another file with two columns, 1=cell_id (cell names), 2=group (class16), file name=spx_bc_class16.group.csv for import in Xenium explorer
sce.meta <- read.csv("spx_bc_class16.meta.csv", h=T, row.names=1, sep=",")
spx_bc <- AddMetaData(spx_bc, sce.meta)

#Figure 4D
Idents(spx_bc) <- "hybridcells"
Cluster_Highlight_Plot(spx_bc, cluster_name = c("Hybrid Cells"), highlight_color = c("#7030A0"), background_color = "lightgray", raster = F, label = F, pt.size = 2) + NoLegend()

#Figure 4E
#Import "spx_bc_class16.group.csv" in Xenum explorer where the same image and data is loaded. Select cell clusters: "THC", "EPCAM & KRT8 only", and "CD16 & CD163 only"
# Screenshot using Windows 'Snipping Tool', a full image (1X), Manually zoom to an area with all three types of cells and snip again at 10x, and again manually Zoom to 100x and snip. 

#Figure S10A
FeaturePlot_scCustom(sce_nd, features = "ERVW-1", raster = F, pt.size = 2, split.by = "SPECIMEN_TYPE")

#Figure S10B
FeaturePlot_scCustom(sce_nd, features = "ERVFRD-1", raster = F, pt.size = 2, split.by = "SPECIMEN_TYPE")

#Figure S10C
FeaturePlot_scCustom(sce_nd_m, features = "ERVW-1", raster = F, pt.size = 2, split.by = "cl295v11SubFull", num_columns = 5)

#Figure S10D
FeaturePlot_scCustom(sce_nd_h, features = "ERVW-1", raster = F, pt.size = 2, split.by = "SPECIMEN_TYPE")

#Figure S10E, syncytin-1 expression data was extracted and cells with (>0) and without (=0) expression were counted in each patient, and patients separated based on those who had at least one cells positive for syncytin-1 (Syncytin+) or none of the cells were positive for syncytin-1 (Syncytin-)

exp_syn <- FetchData(sce_nd, vars = c("ERVW-1", "hybridclass4"))

#Session information
#sessionInfo()
#R version 4.3.0 (2023-04-21 ucrt)
#Platform: x86_64-w64-mingw32/x64 (64-bit)
#Running under: Windows 10 x64 (build 18362)
#
#Matrix products: default
#
#
#locale:
#  [1] LC_COLLATE=English_United States.utf8  LC_CTYPE=English_United States.utf8    LC_MONETARY=English_United States.utf8
#[4] LC_NUMERIC=C                           LC_TIME=English_United States.utf8    
#
#time zone: America/New_York
#tzcode source: internal
#
#attached base packages:
#  [1] stats4    stats     graphics  grDevices utils     datasets  methods   base     
#
#other attached packages:
#  [1] tidyr_1.3.0                 Azimuth_0.5.0               shinyBS_0.61.1              SeuratDisk_0.0.0.9021      
#[5] shiny_1.8.0                 EnhancedVolcano_1.20.0      ggrepel_0.9.4               future_1.33.1              
#[9] fgsea_1.28.0                scds_1.18.0                 scater_1.30.1               scuttle_1.12.0             
#[13] SingleCellExperiment_1.24.0 SummarizedExperiment_1.32.0 Biobase_2.62.0              GenomicRanges_1.54.1       
#[17] GenomeInfoDb_1.38.2         IRanges_2.36.0              S4Vectors_0.40.2            BiocGenerics_0.48.1        
#[21] MatrixGenerics_1.14.0       matrixStats_1.2.0           ggplot2_3.4.4               viridis_0.6.4              
#[25] viridisLite_0.4.2           scCustomize_2.0.1           patchwork_1.1.3             dplyr_1.1.4                
#[29] Seurat_5.0.1                SeuratObject_5.0.1          sp_2.1-3                   
#
#loaded via a namespace (and not attached):
#  [1] R.methodsS3_1.8.2                 progress_1.2.3                    poweRlaw_0.70.6                  
#[4] goftest_1.2-3                     DT_0.31                           Biostrings_2.70.1                
#[7] vctrs_0.6.5                       spatstat.random_3.2-2             digest_0.6.33                    
#[10] png_0.1-8                         shape_1.4.6                       deldir_2.0-2                     
#[13] parallelly_1.36.0                 MASS_7.3-60                       Signac_1.12.0                    
#[16] reshape2_1.4.4                    httpuv_1.6.13                     withr_2.5.2                      
#[19] ggrastr_1.0.2                     ellipsis_0.3.2                    survival_3.5-7                   
#[22] EnsDb.Hsapiens.v86_2.99.0         memoise_2.0.1                     ggbeeswarm_0.7.2                 
#[25] janitor_2.2.0                     systemfonts_1.0.5                 ragg_1.2.7                       
#[28] zoo_1.8-12                        GlobalOptions_0.1.2               gtools_3.9.5                     
#[31] pbapply_1.7-2                     R.oo_1.25.0                       prettyunits_1.2.0                
#[34] rematch2_2.1.2                    KEGGREST_1.42.0                   promises_1.2.1                   
#[37] httr_1.4.7                        restfulr_0.0.15                   rhdf5filters_1.14.1              
#[40] globals_0.16.2                    fitdistrplus_1.1-11               rhdf5_2.46.1                     
#[43] rstudioapi_0.15.0                 miniUI_0.1.1.1                    generics_0.1.3                   
#[46] curl_5.2.0                        zlibbioc_1.48.0                   ScaledMatrix_1.10.0              
#[49] polyclip_1.10-6                   GenomeInfoDbData_1.2.11           SparseArray_1.2.2                
#[52] xtable_1.8-6                      stringr_1.5.1                     pracma_2.4.4                     
#[55] S4Arrays_1.2.0                    BiocFileCache_2.10.1              hms_1.1.3                        
#[58] irlba_2.3.5.1                     colorspace_2.1-0                  filelock_1.0.3                   
#[61] hdf5r_1.3.8                       ROCR_1.0-11                       reticulate_1.34.0                
#[64] spatstat.data_3.0-3               magrittr_2.0.3                    lmtest_0.9-40                    
#[67] snakecase_0.11.1                  readr_2.1.4                       later_1.3.2                      
#[70] lattice_0.22-5                    glmGamPoi_1.14.0                  spatstat.geom_3.2-7              
#[73] future.apply_1.11.1               scattermore_1.2                   XML_3.99-0.16                    
#[76] cowplot_1.1.2                     RcppAnnoy_0.0.21                  pillar_1.9.0                     
#[79] nlme_3.1-164                      caTools_1.18.2                    compiler_4.3.0                   
#[82] beachmat_2.18.0                   RSpectra_0.16-1                   stringi_1.8.3                    
#[85] tensor_1.5                        lubridate_1.9.3                   GenomicAlignments_1.38.0         
#[88] plyr_1.8.9                        crayon_1.5.2                      abind_1.4-7                      
#[91] BiocIO_1.12.0                     googledrive_2.1.1                 bit_4.0.5                        
#[94] fastmatch_1.1-4                   codetools_0.2-19                  textshaping_0.3.7                
#[97] BiocSingular_1.18.0               bslib_0.6.1                       SeuratData_0.2.2.9001            
#[100] paletteer_1.5.0                   plotly_4.10.3                     mime_0.12                        
#[103] splines_4.3.0                     circlize_0.4.15                   Rcpp_1.0.11                      
#[106] fastDummies_1.7.3                 dbplyr_2.4.0                      sparseMatrixStats_1.14.0         
#[109] prismatic_1.1.1                   cellranger_1.1.0                  blob_1.2.4                       
#[112] utf8_1.2.4                        seqLogo_1.68.0                    AnnotationFilter_1.26.0          
#[115] fs_1.6.3                          listenv_0.9.0                     DelayedMatrixStats_1.24.0        
#[118] tibble_3.2.1                      Matrix_1.6-4                      statmod_1.5.0                    
#[121] tzdb_0.4.0                        pkgconfig_2.0.3                   tools_4.3.0                      
#[124] cachem_1.0.8                      RSQLite_2.3.4                     DBI_1.1.3                        
#[127] fastmap_1.1.1                     scales_1.3.0                      grid_4.3.0                       
#[130] ica_1.0-3                         shinydashboard_0.7.2              Rsamtools_2.18.0                 
#[133] sass_0.4.8                        ggprism_1.0.4                     dotCall64_1.1-1                  
#[136] RANN_2.6.1                        farver_2.1.1                      yaml_2.3.8                       
#[139] rtracklayer_1.62.0                cli_3.6.2                         purrr_1.0.2                      
#[142] leiden_0.4.3.1                    lifecycle_1.0.4                   uwot_0.1.16                      
#[145] presto_1.0.0                      BSgenome.Hsapiens.UCSC.hg38_1.4.5 BiocParallel_1.36.0              
#[148] annotate_1.80.0                   timechange_0.2.0                  gtable_0.3.4                     
#[151] rjson_0.2.21                      ggridges_0.5.5                    progressr_0.14.0                 
#[154] parallel_4.3.0                    pROC_1.18.5                       limma_3.58.1                     
#[157] jsonlite_1.8.8                    RcppHNSW_0.5.0                    TFBSTools_1.40.0                 
#[160] bitops_1.0-7                      bit64_4.0.5                       xgboost_1.7.6.1                  
#[163] Rtsne_0.17                        spatstat.utils_3.0-4              BiocNeighbors_1.20.0             
#[166] CNEr_1.38.0                       jquerylib_0.1.4                   shinyjs_2.1.0                    
#[169] R.utils_2.12.3                    lazyeval_0.2.2                    htmltools_0.5.7                  
#[172] GO.db_3.18.0                      sctransform_0.4.1                 rappdirs_0.3.3                   
#[175] ensembldb_2.26.0                  glue_1.6.2                        TFMPvalue_0.0.9                  
#[178] spam_2.10-0                       googlesheets4_1.1.1               XVector_0.42.0                   
#[181] RCurl_1.98-1.13                   BSgenome_1.70.1                   gridExtra_2.3                    
#[184] JASPAR2020_0.99.10                igraph_1.6.0                      R6_2.5.1                         
#[187] forcats_1.0.0                     RcppRoll_0.3.0                    labeling_0.4.3                   
#[190] GenomicFeatures_1.54.1            clipr_0.8.0                       cluster_2.1.6                    
#[193] Rhdf5lib_1.24.1                   gargle_1.5.2                      DirichletMultinomial_1.44.0      
#[196] DelayedArray_0.28.0               tidyselect_1.2.0                  vipor_0.4.5                      
#[199] ProtGenerics_1.34.0               xml2_1.3.6                        AnnotationDbi_1.64.1             
#[202] rsvd_1.0.5                        munsell_0.5.0                     KernSmooth_2.23-22               
#[205] data.table_1.14.10                htmlwidgets_1.6.4                 RColorBrewer_1.1-3               
#[208] biomaRt_2.58.0                    rlang_1.1.2                       spatstat.sparse_3.0-3            
#[211] spatstat.explore_3.2-5            fansi_1.0.6                       beeswarm_0.4.0  
